Package: S4DM
Title: Small Sample Size Species Distribution Modeling
Version: 0.0.2
Authors@R: 
    c(
    person(given = "Brian S.",
           family = "Maitner",
           role = c("aut", "cre"),
           email = "bmaitner@usf.edu",
           comment = c(ORCID = "https://orcid.org/0000-0002-2118-9880")),
           person(given = "Robert L.",
           family = "Richards",
           role = c("aut"),
           email = "r.richards@gatech.edu"),
           person(given = "Ben S.",
           family = "Carlson",
           role = c("aut"),
           email = "ben.s.carlson@gmail.com"),
           person(given = "John M.",
           family = "Drake",
           role = c("aut"),
           email = "john@drakeresearchlab.com"),
           person(given = "Cory",
           family = "Merow",
           role = c("aut"),
           email = "cory.merow@gmail.com")
           )
Description: Implements a set of distribution modeling methods that are suited to species with small sample sizes (e.g., poorly sampled species or rare species). While these methods can also be used on well-sampled taxa, they are united by the fact that they can be utilized with relatively few data points. More details on the currently implemented methodologies can be found in Maitner et al. (2026) <doi:10.1002/ecog.08112>, Drake and Richards (2018) <doi:10.1002/ecs2.2373>, Drake (2015) <doi:10.1098/rsif.2015.0086>, and Drake (2014) <doi:10.1890/ES13-00202.1>.
Depends: R (>= 4.1.0)
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Imports: corpcor, densratio (>= 0.3.1), flexclust, geometry, kernlab,
        maxnet, mvtnorm, np, pROC, robust, rvinecopulib, sf, terra,
        dplyr, Rdpack
Suggests: geodata, BIEN, ggplot2, tidyterra, knitr, testthat, rmarkdown
RdMacros: Rdpack
NeedsCompilation: no
Packaged: 2026-05-09 21:37:29 UTC; bmaitner
Author: Brian S. Maitner [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-2118-9880>),
  Robert L. Richards [aut],
  Ben S. Carlson [aut],
  John M. Drake [aut],
  Cory Merow [aut]
Maintainer: Brian S. Maitner <bmaitner@usf.edu>
Repository: CRAN
Date/Publication: 2026-05-11 06:50:13 UTC
Built: R 4.5.2; ; 2026-05-11 08:03:18 UTC; unix
