# Generated by roxygen2: do not edit by hand

export(ensemble_range_map)
export(evaluate_range_map)
export(fit_density_ratio)
export(fit_plug_and_play)
export(get_env_bg)
export(get_env_pres)
export(make_range_map)
export(project_density_ratio)
export(project_plug_and_play)
export(sdm_threshold)
export(stratify_random)
export(stratify_spatial)
import(geometry)
import(maxnet)
import(sf)
importFrom(Rdpack,reprompt)
importFrom(corpcor,cov.shrink)
importFrom(densratio,KLIEP)
importFrom(densratio,RuLSIF)
importFrom(densratio,uLSIF)
importFrom(dplyr,bind_rows)
importFrom(flexclust,as.kcca)
importFrom(flexclust,kcca)
importFrom(kernlab,ksvm)
importFrom(kernlab,predict)
importFrom(mvtnorm,dmvnorm)
importFrom(np,npudens)
importFrom(np,npudensbw)
importFrom(pROC,auc)
importFrom(pROC,roc)
importFrom(robust,covRob)
importFrom(rvinecopulib,dvinecop)
importFrom(rvinecopulib,vine)
importFrom(sf,st_as_sf)
importFrom(sf,st_crs)
importFrom(sf,st_transform)
importFrom(stats,approx)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,density)
importFrom(stats,fitted)
importFrom(stats,kmeans)
importFrom(stats,na.omit)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(terra,"values<-")
importFrom(terra,"varnames<-")
importFrom(terra,app)
importFrom(terra,buffer)
importFrom(terra,extract)
importFrom(terra,ncell)
importFrom(terra,nlyr)
importFrom(terra,setValues)
importFrom(terra,vect)
importFrom(utils,getFromNamespace)
